@@ -7,6 +7,16 @@ from django.contrib import admin
 from textrank import models
 
 
+class TopicAdmin(admin.ModelAdmin):
+    # save_as = True
+    date_hierarchy = 'updated'
+    list_display = (
+        'name', 'code', 'is_active', 'updated', 'last_editor', 'id',
+    )
+    list_filter = ['is_active', 'updated']
+    search_fields = ['name']
+
+
 class WeightInline(admin.StackedInline):
     model = models.Weight
     extra = 0
@@ -47,6 +57,7 @@ class WeightAdmin(admin.ModelAdmin):
 
 
 register = admin.site.register
+register(models.Topic, TopicAdmin)
 register(models.Group, GroupAdmin)
 register(models.Keyword, KeywordAdmin)
 register(models.Weight, WeightAdmin)
