@@ -28,6 +28,18 @@ INFO = {
     # 'last_updated': None,
 }
 
+# Реестр слов в памяти.
+DICTS = {
+    # Ключевые слова - это одиночные слова.
+    # 'keywords_0': WordsDict(),
+    # Цепочка слов - слова, разделённые нижним подчёркиванием, все эти слова
+    # идут в строгой последовательности друг за другом.
+    # 'chainwords_0': WordsDict(),
+    # Покрытие слов - слова, разделённые знаком +, все эти слова
+    # должны находится в любом месте текста, в любой последовательности.
+    # 'coverwords_0': WordsDict(),
+}
+
 
 class Word(str):
     """
@@ -71,14 +83,15 @@ class WordsDict(dict):
         return word
 
 
-# Ключевые слова - это одиночные слова.
-KEYWORDS = WordsDict('KEYWORDS')
-# Цепочка слов - слова, разделённые нижним подчёркиванием, все эти слова
-# идут в строгой последовательности друг за другом.
-CHAINWORDS = WordsDict('CHAINWORDS')
-# Покрытие слов - слова, разделённые знаком +, все эти слова
-# должны находится в любом месте текста, в любой последовательности.
-COVERWORDS = WordsDict('COVERWORDS')
+def get_dictionaries(topic_id=None):
+    tid = topic_id or 0
+    key1 = 'keywords_%s' % tid
+    key2 = 'chainwords_%s' % tid
+    key3 = 'coverwords_%s' % tid
+    for key in (key1, key2, key3):
+        if key not in DICTS:
+            DICTS[key] = WordsDict(key)
+    return DICTS[key1], DICTS[key2], DICTS[key3]
 
 
 def check_updates():
@@ -102,6 +115,8 @@ def check_updates():
         gid = weight.group_id
         word = Word(weight.keyword)
         word_id = word.object_id
+        topic_id = getattr(weight.group, 'topic_id', None)
+        KEYWORDS, CHAINWORDS, COVERWORDS = get_dictionaries(topic_id)
         # Сначала удаляем из 2-х других словарей старое значение слова.
         # Это необходимо, когда слово было одиночным, а потом изменилось на
         # сочетание или наоборот.
@@ -150,10 +165,6 @@ def check_updates():
 
     utime = (now() - start).total_seconds()
     logger.info('update time = %s sec, count = %d', utime, count)
-    logger.info(
-        'length KEYWORDS = %s, CHAINWORDS = %s, COVERWORDS = %s',
-        len(KEYWORDS), len(CHAINWORDS), len(COVERWORDS)
-    )
     return utime
 
 
@@ -165,7 +176,16 @@ def words_iterator(text):
         yield w.lower()
 
 
-def analyze_text(text, morphology=None):
+def serialize_topic(topic):
+    if topic:
+        return {
+            'id': topic.id,
+            'code': topic.code,
+            'name': topic.name,
+        }
+
+
+def analyze_text(text, morphology=None, topic=None):
     if morphology is None:
         # Для ветки master доступно указание языка.
         # morphology = MorphAnalyzer(lang=get_language()).parse
@@ -181,11 +201,15 @@ def analyze_text(text, morphology=None):
     # Слова должны быть в оригинальной последовательности.
     words = list(words_iterator(text))
 
+    topic_id = topic.id if topic else None
+    KEYWORDS, CHAINWORDS, COVERWORDS = get_dictionaries(topic_id)
+
     if not GROUPS or not (KEYWORDS or CHAINWORDS or COVERWORDS):
         logger.warn('no groups and no key|chain|cover words')
         atime = (now() - start).total_seconds()
         logger.info('analyze time = %s', atime)
         return {
+            'topic': serialize_topic(topic),
             'group': {},
             'found': {},
             'words': words,
@@ -300,6 +324,7 @@ def analyze_text(text, morphology=None):
     atime = (now() - start).total_seconds()
     logger.info('analyze time = %s', atime)
     return {
+        'topic': serialize_topic(topic),
         'group': group,
         'found': found,
         'words': words,
