@@ -15,10 +15,10 @@ from django.utils.translation import gettext_lazy as _
 from textrank.validators import keyword_validator
 
 
-class Group(models.Model):
-    """Модель группы ключевых слов."""
-    created = models.DateTimeField(_('создана'), auto_now_add=True)
-    updated = models.DateTimeField(_('обновлена'), auto_now=True)
+class Topic(models.Model):
+    """Раздел для группировки ранжирования."""
+    created = models.DateTimeField(_('создан'), auto_now_add=True)
+    updated = models.DateTimeField(_('обновлён'), auto_now=True)
     name = models.CharField(
         _('название'), max_length=150, unique=True,
         help_text=_('Обязательно. 150 символов и менее.'),
@@ -30,7 +30,60 @@ class Group(models.Model):
         _('описание'), blank=True, db_index=True,
     )
     code = models.CharField(
-        _('код группы'), max_length=32, default=get_random_string, unique=True,
+        _('код раздела'), max_length=32, default=get_random_string, unique=True,
+        help_text=_('Обязательно. 32 символа и менее, уникальное значение.'),
+        error_messages={
+            'unique': _('Такой код уже существует.'),
+        },
+    )
+    is_active = models.BooleanField(
+        _('активный'), default=True, db_index=True, help_text=_(
+            'Отключите, если раздел не используется для анализа.'
+        )
+    )
+    # Пользователь, который последним изменил объект.
+    last_editor = models.ForeignKey(
+        settings.AUTH_USER_MODEL,
+        null=True, blank=True,
+        editable=False,
+        verbose_name=_('последний редактор'),
+        on_delete=models.PROTECT,
+    )
+
+    class Meta:
+        verbose_name = _('раздел ранжирования')
+        verbose_name_plural = _('разделы ранжирования')
+        ordering = ('name',)
+
+    def __str__(self):
+        return self.name
+
+    def delete(self, **kwargs):
+        # Удалять объекты в Django запрещено. Только из базы данных.
+        self.is_active = False
+        self.save(**kwargs)
+
+    def get_description(self):
+        return self.description.replace('\r\n', ' ').replace('\n', ' ')
+
+
+class Group(models.Model):
+    """Модель группы ключевых слов."""
+    created = models.DateTimeField(_('создана'), auto_now_add=True)
+    updated = models.DateTimeField(_('обновлена'), auto_now=True)
+
+    name = models.CharField(
+        _('название'), max_length=150,
+        help_text=_('Обязательно. 150 символов и менее.'),
+        error_messages={
+            'unique': _('Такое название уже существует.'),
+        },
+    )
+    description = models.TextField(
+        _('описание'), blank=True, db_index=True,
+    )
+    code = models.CharField(
+        _('код группы'), max_length=32, default=get_random_string,
         help_text=_('Обязательно. 32 символа и менее, уникальное значение.'),
         error_messages={
             'unique': _('Такой код уже существует.'),
@@ -41,6 +94,12 @@ class Group(models.Model):
             'Отключите, если группа не используется для анализа.'
         )
     )
+    topic = models.ForeignKey(
+        Topic,
+        null=True, blank=True,
+        verbose_name=_('раздел'),
+        on_delete=models.PROTECT,
+    )
     # Пользователь, который последним изменил объект.
     last_editor = models.ForeignKey(
         settings.AUTH_USER_MODEL,
@@ -54,6 +113,24 @@ class Group(models.Model):
         verbose_name = _('группа ключевых слов')
         verbose_name_plural = _('группы ключевых слов')
         ordering = ('name',)
+        constraints = [
+            models.UniqueConstraint(
+                fields=['name'], name='group_name_default_unique',
+                condition=models.Q(topic__isnull=True),
+            ),
+            models.UniqueConstraint(
+                fields=['name', 'topic'], name='group_name_topic_unique',
+                condition=models.Q(topic__isnull=False),
+            ),
+            models.UniqueConstraint(
+                fields=['code'], name='group_code_default_unique',
+                condition=models.Q(topic__isnull=True),
+            ),
+            models.UniqueConstraint(
+                fields=['code', 'topic'], name='group_code_topic_unique',
+                condition=models.Q(topic__isnull=False),
+            ),
+        ]
 
     def __str__(self):
         return self.name
@@ -92,7 +169,7 @@ class Keyword(models.Model):
     created = models.DateTimeField(_('создано'), auto_now_add=True)
     updated = models.DateTimeField(_('обновлено'), auto_now=True)
     word = models.CharField(
-        _('слово'), max_length=100, unique=True,
+        _('слово'), max_length=100,
         help_text=_(
             'Обязательно. 100 символов и менее. '
             'Значение может содержать только буквы в нижнем регистре, '
@@ -111,6 +188,12 @@ class Keyword(models.Model):
             'Отключите, если слово не используется для анализа.'
         )
     )
+    topic = models.ForeignKey(
+        Topic,
+        null=True, blank=True,
+        verbose_name=_('раздел'),
+        on_delete=models.PROTECT,
+    )
     # Пользователь, который последним изменил объект.
     last_editor = models.ForeignKey(
         settings.AUTH_USER_MODEL,
@@ -129,6 +212,16 @@ class Keyword(models.Model):
         verbose_name = _('ключевое слово')
         verbose_name_plural = _('ключевые слова')
         ordering = ('word',)
+        constraints = [
+            models.UniqueConstraint(
+                fields=['word'], name='keyword_default_unique',
+                condition=models.Q(topic__isnull=True),
+            ),
+            models.UniqueConstraint(
+                fields=['word', 'topic'], name='keyword_topic_unique',
+                condition=models.Q(topic__isnull=False),
+            ),
+        ]
 
     def __str__(self):
         return self.word
