@@ -4,7 +4,9 @@
 # This file is distributed under the BSD 3-Clause License.
 #
 from django.urls import path
-from textrank.views import index, groups, keywords, rank, weightform, samples
+from textrank.views import (
+    index, groups, keywords, rank, weightform, samples, topics
+)
 
 app_name = 'textrank'
 
@@ -21,4 +23,6 @@ urlpatterns = [
     path('weightform/<int:id>/', weightform, name='change_weight'),
     path('samples/', samples, name='samples'),
     path('samples/<int:id>/', samples, name='sample'),
+    path('topics/', topics, name='topics'),
+    path('topics/<int:id>/', topics, name='topic'),
 ]
